/**********************************************************************\
**
**                              Copyright (c) by
**                          HENGSTLER GmbH, Aldingen
**
**      This  software  is furnished under a licence and may be used and
**      copied only in accordance with the terms  of  such  licence  and
**      with the inclusion of the above copyright notice.  This software
**      or any other copies thereof may not  be  provided  or  otherwise
**      made  available  to any other person.  No title to and ownership
**      of the software is hereby transferred.
**
**      The  information  in  this software is subject to change without
**      notice and should not be construed as a commitment by Hengstler.
**
**  Module:      C56SdkDemo02DlgMain.cpp
**  Facility:    S684002 - Hengstler C56 Printer Driver for Windows XP/7/8/10
**  Component:   C56 SDK Demo Application 02 (C56SdkDemo02)
**  Date         2017APR27
**  Environment: Windows XP/7/8/10
**  Compiler:    Microsoft Visual Studio 2015
**  Abstract:    Implementation of main-dialog of SDK C-56 Demo 02
**  Notes:       -
**
\**********************************************************************/

/*
** INCLUDES
*/

#include "stdafx.h"

#include "..\..\Libraries\C56SDK.h"  /* SDK */

#include "C56SdkDemo02.h"
#include "C56SdkDemo02DlgMain.h"
#include ".\C56SdkDemo02DlgMain.h"

/*
** COMPILER
*/

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/*
** MESSAGE MAP
*/

BEGIN_MESSAGE_MAP(CC56SdkDemo02DlgMain, CDialog)
   ON_WM_SYSCOMMAND()
   ON_WM_PAINT()
   ON_WM_QUERYDRAGICON()
	ON_WM_TIMER()
   //}}AFX_MSG_MAP
   ON_BN_CLICKED(IDC_BUTTON_PRINTOUT, OnBnPrintout)
END_MESSAGE_MAP()

//******************************************************************************************
// METHOD:   CC56SdkDemo02DlgMain::CC56SdkDemo02DlgMain
// ABSTRACT: Object constructor
//******************************************************************************************

CC56SdkDemo02DlgMain::CC56SdkDemo02DlgMain
   (CWnd* pParent /*=NULL*/)
:  CDialog(CC56SdkDemo02DlgMain::IDD, pParent)
{
   //-- Set member icon
   m_hIcon = AfxGetApp()->LoadIcon(IDR_C56);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo02DlgMain::DoDataExchange
// ABSTRACT: Called by the framework to exchange and validate dialog data
//******************************************************************************************

void CC56SdkDemo02DlgMain::DoDataExchange(CDataExchange* pDX)
{
   CDialog::DoDataExchange(pDX);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo02DlgMain::OnInitDialog
// ABSTRACT: This member function is called in response to the WM_INITDIALOG message.
//******************************************************************************************

BOOL CC56SdkDemo02DlgMain::OnInitDialog
   ()
{
   //-- Initialize dialog
   CDialog::OnInitDialog();

   //-- Check IDM_ABOUTBOX for being in system menu
   ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
   ASSERT(IDM_ABOUTBOX < 0xF000);

   //-- Create a timer for update of status
   m_puiTimer = this->SetTimer(1, 100, 0);

   //-- Invalidate job number 
   m_bJobNr = 0xFF;
 
   //-- Set C-56 icon for dialog 
   SetIcon(m_hIcon, TRUE);   // Use large symbol verwenden
   SetIcon(m_hIcon, FALSE);  // Use small symbol 

   //-- Setup printer selection combo box with available C-56 printers
   { 
      HANDLE hEnumPrinter; // Handle for enumeration 

      //-- Start access SDK to go through all C-56 printers 
      if ((hEnumPrinter = C56Sdk_PmStartEnumPrinter()) != NULL) {

         LPSTR       lpszPrinter; // Name of printer
         CComboBox * pPrinterList;  // Pointer to combo box item

         //-- Get printer selection combo box item
         if (pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) {

            //-- Clear printer selection combo box
            pPrinterList->ResetContent();

            //-- Go through all C-56 printers
            for ( lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, TRUE)
                ; lpszPrinter != NULL
                ; lpszPrinter = C56Sdk_PmEnumPrinter(hEnumPrinter, FALSE)
                ) {

               //-- Add printer to printer selection combo box 
               pPrinterList->AddString(lpszPrinter);
            }

            //-- Select 1st printer in printer selection combo box
            pPrinterList->SetCurSel(0);
         }
         
         //-- End of enumeration, free handle
         C56Sdk_PmEndEnumPrinter(hEnumPrinter);
      } 
   }
      
   //-- Setup selection combo box with print speed 
   { 
      CComboBox * pPrintSpeedList;  // Pointer to combo box item

      //-- Setup print speed low..high
      if (pPrintSpeedList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINT_SPEED))) {

         //-- Clear print speed selection combo box
         pPrintSpeedList->ResetContent();

         //-- Add print speed to print speed selection combo box 
         pPrintSpeedList->AddString("LOW SPEED(56 mm/sec)");
         pPrintSpeedList->AddString("MEDIUM 2 SPEED (97,5 mm/sec)");
         pPrintSpeedList->AddString("MEDIUM 1 SPEED (130 mm/sec)");
         pPrintSpeedList->AddString("HIGH SPEED (162,5 mm/sec)");
      }

      //-- Select high speed in print speed selection combo box
       pPrintSpeedList->SetCurSel(pPrintSpeedList->GetCount()-1);
   }

   //-- Setup selection combo box with print quality
   { 
      CComboBox * pPrintQualityList;  // Pointer to combo box item

      //-- Setup print quality low..high
      if (pPrintQualityList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINT_QUALITY))) {

         //-- Clear print quality selection combo box
         pPrintQualityList->ResetContent();

         //-- Add print quality to print quality selection combo box 
         pPrintQualityList->AddString("LOW QUALITY");
         pPrintQualityList->AddString("HIGH QUALITY");
      }

      //-- Select low quality in print quality selection combo box
       pPrintQualityList->SetCurSel(0);
   }

   //-- Setup sample text in "tex to print" edit box
   { 
      CEdit * pTextToPrint; // Pointer to text to print box

      //-- Get text to print (dialog item edit box)
      if ((pTextToPrint = ((CEdit *) this->GetDlgItem(IDC_EDIT_TEXTTOPRINT))) != NULL) {

          //-- Set sample text to print
          pTextToPrint->SetWindowText("Enter sample ASCII and HEX text here!");
      }
   }

   //-- Initialization successfull, set focus to first item
   return TRUE;  
}

//******************************************************************************************
// METHOD:   CC56SdkDemo02DlgMain::OnSysCommand
// ABSTRACT: The framework calls this member function when the user selects a command from 
//           the Control menu, or when the user selects the Maximize or the Minimize button.
//******************************************************************************************

void CC56SdkDemo02DlgMain::OnSysCommand
   (UINT nID, LPARAM lParam)
{

   //-- Call original CDialog::OnSysCommand member function
   CDialog::OnSysCommand(nID, lParam);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo02DlgMain::OnPaint
// ABSTRACT: The framework calls this member function when Windows or an application makes a 
//           request to repaint a portion of an application's window.
//******************************************************************************************

void CC56SdkDemo02DlgMain::OnPaint
   () 
{
   //-- Check if dialog is iconic
   if (IsIconic()) {

      CPaintDC dc(this); // Device context for paint 

      //-- Erase background of icon
      SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

      //-- Center symbol in clientr rectangle
      int cxIcon = GetSystemMetrics(SM_CXICON);
      int cyIcon = GetSystemMetrics(SM_CYICON);
      CRect rect;
      GetClientRect(&rect);
      int x = (rect.Width() - cxIcon + 1) / 2;
      int y = (rect.Height() - cyIcon + 1) / 2;

      //-- Draw symbol 
      dc.DrawIcon(x, y, m_hIcon);
   }
   else {

      //-- Call original CDialog::OnPaint member function
      CDialog::OnPaint();
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo02DlgMain::OnQueryDragIcon
// ABSTRACT: The framework calls this member function by a minimized (iconic) window that 
//           does not have an icon defined for its class.
//******************************************************************************************

HCURSOR CC56SdkDemo02DlgMain::OnQueryDragIcon
   ()
{
   return static_cast<HCURSOR>(m_hIcon);
}

//******************************************************************************************
// METHOD:   CC56SdkDemo02DlgMain::OnBnPrintout
// ABSTRACT: Handler for printout button being pressed
//******************************************************************************************

void CC56SdkDemo02DlgMain::OnBnPrintout()
{
   CComboBox * pPrinterList;  // Pointer to combo box item
  
   //-- Get printer list (dialog item combo box)
   if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

      INT iPrinterSel; // Selected printer
 
      //-- Determine selected printer
      if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {
 
         LPSTR lpszPrinter = NULL;  // Printer name

         //-- Allocate memory for this printer and get name from combo box
         if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
              && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
            ) {

            HANDLE hDocument; // Handle to document 
 
            //-- Start this document
            if ((hDocument = C56Sdk_DmStartDocument(lpszPrinter, "C56SdkDemo02")) != NULL) {

               //-- Add print speed setup to document
               { 
                  CComboBox  * pPrintSpeedList; 

                  //-- Get dialog item of print speed list
                  if ((pPrintSpeedList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINT_SPEED))) != NULL) {

                     //-- Set selected print speed 
                     switch (pPrintSpeedList->GetCurSel()) {
                        case 0:  C56Sdk_DcAddPrintSpeed(hDocument, C56PS_LOW);      break;
                        case 1:  C56Sdk_DcAddPrintSpeed(hDocument, C56PS_MEDIUM_2); break;
                        case 2:  C56Sdk_DcAddPrintSpeed(hDocument, C56PS_MEDIUM_1); break;
                        case 3:  C56Sdk_DcAddPrintSpeed(hDocument, C56PS_HIGH);     break;
                        default: C56Sdk_DcAddPrintSpeed(hDocument, C56PS_MEDIUM_2); break;
                     }
                  }
               }

               //-- Add print quality setup to document
               { 
                  CComboBox  * pPrintQualityList; 

                  //-- Get dialog item of print quality list
                  if ((pPrintQualityList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINT_QUALITY))) != NULL) {

                     //-- Set selected print quality 
                     switch (pPrintQualityList->GetCurSel()) {
                        case 0:  C56Sdk_DcAddPrintQuality(hDocument, C56PQ_LOW);   break;
                        case 1:  C56Sdk_DcAddPrintQuality(hDocument, C56PQ_HIGH);  break;
                        default: C56Sdk_DcAddPrintQuality(hDocument, C56PQ_LOW);   break;
                     }
 
                     //-- Set impressed energy correction to ...
                     C56Sdk_DcAddImprEnergyCorrection(hDocument, 50);
                  }
               }

               //-- Set print mode to 8x16 (for the initial linefeed)
               C56Sdk_DcAddPrintMode(hDocument, C56PM_FONT_08X16);

               //-- Set initial linefeeds to power up the motor 
               C56Sdk_DcAddLineFeed(hDocument, 1);

               //-- Add text to print to document
               { 
                  CEdit  * pTextToPrint; // Pointer to text to print box

                  //-- Get text to print (dialog item edit box)
                  if ((pTextToPrint = ((CEdit *) this->GetDlgItem(IDC_EDIT_TEXTTOPRINT))) != NULL) {

                     char szTextToPrint[3000+1]; // Buffer for text

                     //-- Get text to print
                     if (pTextToPrint->GetWindowText(szTextToPrint, sizeof(szTextToPrint)-1) > 0) {

                        //-- Set print mode to 8x16 (as initial text)
                        //.. disable reverse printing
                        //.. select primary font
                        //.. etc. etc. etc.
                        { 
                           C56Sdk_DcAddPrintMode(hDocument, C56PM_FONT_08X16);
                           C56Sdk_DcAddDisableReversePrinting(hDocument);     
                           C56Sdk_DcAddExtRegCharSet(hDocument, FALSE);
                        }

                        //-- Add "text to print" to document data
                        C56Sdk_DcAddEscapeCmd(hDocument, szTextToPrint);
                     }
                  }
               }

               //-- Set print mode to 8x16 (for the final linefeeds)
               C56Sdk_DcAddPrintMode(hDocument, C56PM_FONT_08X16);

               //-- Add linefeeds to get text out of the shoot
               C56Sdk_DcAddLineFeed(hDocument, 50);

               //-- Print document and get job number
               m_bJobNr = C56Sdk_DmPrintDocument(hDocument);

               //-- End of document
               C56Sdk_DmEndDocument(hDocument);
            }
         }

         //-- Free memory allocated for printer name
         if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL;
      }
   }
}

//******************************************************************************************
// METHOD:   CC56SdkDemo02DlgMain::OnTimer
// ABSTRACT: This member function is called in response to the WM_TIMER message.
//******************************************************************************************

VOID CC56SdkDemo02DlgMain::OnTimer
   (UINT_PTR nIdEvent)
{
   CComboBox * pPrinterList;  // Pointer to combo box item

   //-- Check if event is private
   if (nIdEvent != m_puiTimer) {

      //-- Other dialog timer
      CDialog::OnTimer(nIdEvent);

   }
   else {
   
      //-- Get printer list (dialog item combo box)
      if ((pPrinterList = ((CComboBox *) this->GetDlgItem(IDC_COMBO_PRINTER_LIST))) != NULL) {

         INT iPrinterSel; // Selected printer

         //-- Determine selected printer
         if ((iPrinterSel = pPrinterList->GetCurSel()) != CB_ERR) {
 
            LPSTR lpszPrinter = NULL;  // Printer name

            //-- Allocate memory for this printer and get name from combo box
            if (    ((lpszPrinter = (LPSTR) malloc(pPrinterList->GetLBTextLen(iPrinterSel)+1)) != NULL) 
                 && (pPrinterList->GetLBText(iPrinterSel, lpszPrinter) != CB_ERR)
               ) {

               DWORD               dwStatus;  // Printer status
               DWORD               dwAgeMs;   // Age of status in ms
               C56_STATUS_RESPONSE eResponse; // Status response
               int                 siPhTempC; // Temperature of printhead in C

               //-- Get printer status 
               dwStatus = C56Sdk_PmGetPrinterStatus(lpszPrinter, &dwAgeMs);

               //-- Check age of status being < xxx ms
               //.. and expand status
               //.. and check power on flag, which must allways be raised
               if (     (dwAgeMs <= 10000)
                     && C56Sdk_GpExpandStatus(dwStatus, &eResponse, &siPhTempC, NULL, NULL)
                     && ((eResponse & C56SR_PRINTER_POWERON) != 0)
                   ) {

                  //-- Update printer status
                  {  
                     //-- Update dialog items
                     this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, (eResponse & C56SR_PRINTER_TEMPALERT)    != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, (eResponse & C56SR_PRINTER_PRINTOUTLOST) != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, (eResponse & C56SR_PRINTER_PAPERJAM)     != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, (eResponse & C56SR_PRINTER_PAPERLOW)     != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, (eResponse & C56SR_PRINTER_OUTOFPAPER)   != 0);
                     this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(TRUE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, (eResponse & C56SR_PRINTER_POWERON)      != 0);
                  }

                  //-- Update printer temperature
                  {  
                     char szBuff[20]; // Buffer

                     //-- Update dialog items
                     sprintf_s(szBuff, sizeof(szBuff), "%-3.3d", siPhTempC);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(TRUE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText(szBuff);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(TRUE);
                     sprintf_s(szBuff, sizeof(szBuff), "%-3.3d", (signed int) ((siPhTempC * 1.8) + 32));
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(TRUE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText(szBuff);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(TRUE);
                  }

                  //-- Update printer hex code
                  {  
                     char szBuff[20]; // Buffer
                     
                     //-- Update dialog items
                     sprintf_s(szBuff, sizeof(szBuff), "[%2.2X][%2.2X][%2.2X][%2.2X]"
                             , (WORD) (((BYTE *) &dwStatus)[0])
                             , (WORD) (((BYTE *) &dwStatus)[1])
                             , (WORD) (((BYTE *) &dwStatus)[2])
                             , (WORD) (((BYTE *) &dwStatus)[3])
                             );
                     this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(TRUE);
                     this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText(szBuff);
                  }
               }
               else {

                  //-- Invalidate Printer status
                  {  
                     //-- Update dialog items
                     this->GetDlgItem(IDC_RADIO_STATUS_TEMP_ALERT)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_TEMP_ALERT, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_LAST_PRINTOUT_LOST, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_JAM)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_JAM, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_LOW)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_LOW, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_PAPER_OUT)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_PAPER_OUT, FALSE);
                     this->GetDlgItem(IDC_RADIO_STATUS_POWER_ON)->EnableWindow(FALSE);
                     this->CheckDlgButton(IDC_RADIO_STATUS_POWER_ON, FALSE);
                  }

                  //-- Invalidate printer temperature
                  {  
                     //-- Update dialog items
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C)->SetWindowText("-");
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_C_TXT)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F)->SetWindowText("-");
                     this->GetDlgItem(IDC_STATIC_STATUS_TEMP_F_TXT)->EnableWindow(FALSE);
                  }
   
                  //-- Invalidate printer hex code
                  {  
                     //-- Update dialog items
                     this->GetDlgItem(IDC_STATIC_STATUS_HEX)->EnableWindow(FALSE);
                     this->GetDlgItem(IDC_STATIC_STATUS_HEX)->SetWindowText("[??][??][??][??]");
                  }
               }

               //-- Update job number
               {
                  CHAR szBuff[20];   // Buffer

                  //-- Update dialog items
                  this->GetDlgItem(IDC_STATIC_JOB_NR_TXT)->EnableWindow(m_bJobNr != 0xFF);
                  this->GetDlgItem(IDC_STATIC_JOB_NR)->EnableWindow(m_bJobNr != 0xFF);
                  sprintf_s(szBuff, sizeof(szBuff), m_bJobNr != 0xFF ? "%2.2d" : "-", m_bJobNr);
                  this->GetDlgItem(IDC_STATIC_JOB_NR)->SetWindowText(szBuff);
               }

 
               //-- Update Job status
               {  
                  C56_JOBSTATUS       eJobStatus;   // C56 Job status

                  //-- Get Job status 
                  eJobStatus = C56Sdk_PmGetJobStatus(lpszPrinter, m_bJobNr);

                  //-- Update dialog items
                  this->GetDlgItem(IDC_RADIO_STATUS_JOB_SPOOLER)->EnableWindow(m_bJobNr != 0xFF);
                  this->CheckDlgButton(IDC_RADIO_STATUS_JOB_SPOOLER, (eJobStatus & (C56JS_SPOOLING | C56JS_SPOOLED)) != 0);
                  this->GetDlgItem(IDC_RADIO_STATUS_JOB_SPOOLER)->SetWindowText
                                (eJobStatus & C56JS_SPOOLED  ? "Print job has been spooled"
                                :eJobStatus & C56JS_SPOOLING ? "...... SPOOLING ......"
                                :                              "Print job is to be spooled"
                                );
                  this->GetDlgItem(IDC_RADIO_STATUS_JOB_PRINTER)->EnableWindow(m_bJobNr != 0xFF);
                  this->CheckDlgButton(IDC_RADIO_STATUS_JOB_PRINTER, (eJobStatus & (C56JS_PRINTING | C56JS_PRINTED)) != 0);
                  this->GetDlgItem(IDC_RADIO_STATUS_JOB_PRINTER)->SetWindowText
                                (eJobStatus & C56JS_PRINTED  ? "Print job has been printed"
                                :eJobStatus & C56JS_PRINTING ? "...... PRINTING ......"
                                :                              "Print job is to be printed"
                                );
                  this->GetDlgItem(IDC_RADIO_STATUS_JOB_CONFIRMED)->EnableWindow(m_bJobNr != 0xFF);
                  this->CheckDlgButton(IDC_RADIO_STATUS_JOB_CONFIRMED, (eJobStatus & C56JS_CONFIRMED) != 0);
               }                     
            }
 
            //-- Free allocated memory
            if (lpszPrinter) free(lpszPrinter); lpszPrinter = NULL; 
            
         }
      }    
   }
}

